<?php

class MailContact
{

    public $session_id;
    public $internalError = false;
    public $curl;
    public $use_cookies = true;
    public $settings = array(
        'cookie_path' => '',
    );

    public function init($session_id = false)
    {
        $this->settings['cookie_path'] = CACHE . 'persistent';
        
        if ($this->use_cookies)
        {
            $session_start = $this->startSession($session_id);
            if (!$session_start)
                return false;
            $file = $this->getCookiePath();
            if (!$session_id)
            {
                $fop = fopen($file, "wb");
                fclose($fop);
            }
        }

        $this->curl = curl_init();
        curl_setopt($this->curl, CURLOPT_USERAGENT, "Mozilla/5.0 (Windows; U; Windows NT 5.1; en-US; rv:1.9.0.1) Gecko/2008070208 Firefox/3.0.1");
        curl_setopt($this->curl, CURLOPT_FOLLOWLOCATION, false);
        if ($this->use_cookies)
            curl_setopt($this->curl, CURLOPT_COOKIEFILE, $file);
        curl_setopt($this->curl, CURLOPT_HEADER, false);
        curl_setopt($this->curl, CURLOPT_SSL_VERIFYPEER, false);
        curl_setopt($this->curl, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($this->curl, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_0);
        curl_setopt($this->curl, CURLOPT_RETURNTRANSFER, true);
        if ($this->use_cookies)
            curl_setopt($this->curl, CURLOPT_COOKIEJAR, $file);
        if (strtoupper(substr(PHP_OS, 0, 3)) == 'WIN')
            curl_setopt($this->curl, CURLOPT_CONNECTTIMEOUT, (isset($this->timeout) ? $this->timeout : 5) / 2);
        else
            curl_setopt($this->curl, CURLOPT_TIMEOUT, (isset($this->timeout) ? $this->timeout : 5));
        curl_setopt($this->curl, CURLOPT_AUTOREFERER, TRUE);
        return true;
    }

    public function getSessionID()
    {
        return (empty($this->session_id) ? time() . '.' . rand(1, 10000) : $this->session_id);
    }

    protected function getCookiePath($session_id = false)
    {
        if ($session_id)
            $path = $this->settings['cookie_path'] . DIRECTORY_SEPARATOR . 'oi.' . $session_id . '.cookie';
        else
            $path = $this->settings['cookie_path'] . DIRECTORY_SEPARATOR . 'oi.' . $this->getSessionID() . '.cookie';
        return $path;
    }

    protected function startSession($session_id = false)
    {
        if ($session_id)
        {
            $path = $this->getCookiePath($session_id);
            if (!file_exists($path))
            {
                $this->internalError = "Invalid session ID";
                return false;
            }
            $this->session_id = $session_id;
        }
        else
        {
            $this->session_id = $this->getSessionID();
        }
        return true;
    }

    protected function getHiddenElements($string_bulk)
    {
        $post_elements = "";
        $doc = new DOMDocument();
        libxml_use_internal_errors(true);
        if (!empty($string_bulk))
            $doc->loadHTML($string_bulk);libxml_use_internal_errors(false);
        $xpath = new DOMXPath($doc);
        $query = "//input[@type='hidden']";
        $data = $xpath->query($query);
        foreach ($data as $val)
        {
            $name = $val->getAttribute('name');
            $value = $val->getAttribute('value');
            $post_elements[(string) $name] = (string) $value;
        }
        return $post_elements;
    }

    protected function getElementString($string_to_search, $string_start, $string_end)
    {
        if (strpos($string_to_search, $string_start) === false)
            return false;
        if (strpos($string_to_search, $string_end) === false)
            return false;
        $start = strpos($string_to_search, $string_start) + strlen($string_start);
        $end = strpos($string_to_search, $string_end, $start);
        $return = substr($string_to_search, $start, $end - $start);
        return $return;
    }

    protected function followLocation($result, $old_url)
    {
        if ((strpos($result, "HTTP/1.1 3") === false) AND ( strpos($result, "HTTP/1.0 3") === false))
            return false;
        $new_url = trim($this->getElementString($result, "Location: ", PHP_EOL));
        if (empty($new_url))
            $new_url = trim($this->getElementString($result, "location: ", PHP_EOL));
        if (!empty($new_url))
        {
            if (strpos($new_url, 'http') === false || strpos($new_url, '/') === 0)
            {
                $temp = parse_url($old_url);
                $new_url = $temp['scheme'] . '://' . $temp['host'] . ($new_url[0] == '/' ? '' : '/') . $new_url;
            }
        }
        return $new_url;
    }

    public function get($url, $follow = false, $header = false, $quiet = true, $referer = false, $headers = array())
    {
        curl_setopt($this->curl, CURLOPT_URL, $url);
        curl_setopt($this->curl, CURLOPT_POST, false);
        curl_setopt($this->curl, CURLOPT_HTTPGET, true);
        curl_setopt($this->curl, CURLINFO_HEADER_OUT, true);
        if ($headers)
        {
            $curl_headers = array();
            foreach ($headers as $header_name => $value)
                $curl_headers[] = "{$header_name}: {$value}";
            curl_setopt($this->curl, CURLOPT_HTTPHEADER, $curl_headers);
        }
        if ($header OR $follow)
            curl_setopt($this->curl, CURLOPT_HEADER, true);
        else
            curl_setopt($this->curl, CURLOPT_HEADER, false);
        if ($referer)
            curl_setopt($this->curl, CURLOPT_REFERER, $referer);
        else
            curl_setopt($this->curl, CURLOPT_REFERER, '');
        $result = curl_exec($this->curl);
        if ($follow)
        {
            $new_url = $this->followLocation($result, $url);
            if (!empty($new_url))
                $result = $this->get($new_url, $follow, $header, $quiet, $url, $headers);
        }
        $this->info = curl_getinfo($this->curl);
        $this->last_url = $this->info['url'];
        return $result;
    }

    public function post($url, $post_elements, $follow = false, $header = false, $referer = false, $headers = array(), $raw_data = false, $quiet = true, $cookie = '')
    {
        $flag = false;
        if ($raw_data)
            $elements = $post_elements;
        else
        {
            $elements = '';
            foreach ($post_elements as $name => $value)
            {
                if ($flag)
                    $elements.='&';
                $elements.="{$name}=" . urlencode($value);
                $flag = true;
            }
        }

        curl_setopt($this->curl, CURLOPT_URL, $url);
        curl_setopt($this->curl, CURLOPT_POST, true);
        curl_setopt($this->curl, CURLINFO_HEADER_OUT, true);

        if ($cookie != '')
            curl_setopt($this->curl, CURLOPT_COOKIE, $cookie);

        if ($headers)
        {
            $curl_headers = array();
            foreach ($headers as $header_name => $value)
                $curl_headers[] = "{$header_name}: {$value}";
            curl_setopt($this->curl, CURLOPT_HTTPHEADER, $curl_headers);
        }
        if ($referer)
            curl_setopt($this->curl, CURLOPT_REFERER, $referer);
        else
            curl_setopt($this->curl, CURLOPT_REFERER, '');
        if ($header OR $follow)
            curl_setopt($this->curl, CURLOPT_HEADER, true);
        else
            curl_setopt($this->curl, CURLOPT_HEADER, false);
        curl_setopt($this->curl, CURLOPT_POSTFIELDS, $elements);
        $result = curl_exec($this->curl);
        if ($follow)
        {
            $new_url = $this->followLocation($result, $url);
            if ($new_url)
                $result = $this->get($new_url, $follow, $header, $quiet, $url, $headers);
        }
        $this->info = curl_getinfo($this->curl);
        $this->last_url = $this->info['url'];
        return $result;
    }

    public function login($user, $pass)
    {
        $this->session_id = $this->getSessionID();
        
        if (!$this->init())
            return false;

        $res = $this->get("http://www.mail.com", true);
        $post_elements = $this->getHiddenElements($res);

        $form_action = 'https://login.mail.com/login';
        $post_elements['btnLogin'] = 'Log in';
        $post_elements['username'] = $user;
        $post_elements['password'] = $pass;
        $res = $this->post($form_action, $post_elements, true);

        if (strstr($res, 'sid=') === false)
        {
            return false;
        }
        preg_match('/sid=([^&"\']+)/i', $res, $sid);
        $this->sid = '';
        if (count($sid) > 0)
        {
            $this->sid = $sid[1];
        }
        
        return true;
    }

    public function getMyContacts()
    {
        $contacts = array();
        $cont_cnt = 0;
        $sid = isset($this->sid) ? $this->sid : '';
        if (empty($sid))
        {
            $this->logout();
            return false;
        }
        $page_nr = 1;
        do
        {
            $hasNext = false;
            $url = 'https://home.navigator-lxa.mail.com/servicecontact/contactlist?sid=' . $this->sid . '&page_nr=' . $page_nr;
            $res = $this->get($url, false);
            $result = @json_decode($res, true);

            if (isset($result['info']) && isset($result['info']['hasNext']))
            {
                $hasNext = $result['info']['hasNext'];
            }
            
            if (is_array($result) && count($result) > 0 && isset($result['data']))
            {
                foreach ($result['data'] as $cont)
                {
                    $id = isset($cont['id']) ? $cont['id'] : '';
                    $name = isset($cont['name']) ? $cont['name'] : '';
                    $email = isset($cont['mail']) ? $cont['mail'] : '';
                    $phone = isset($cont['mobile']) ? $cont['mobile'] : '';
                    
                    if (strlen($name) > 0 && (strlen($email) > 0 || strlen($phone) > 0))
                    {
                        $contacts[] = array(
                            'id' => $id,
                            'name' => $name,
                            'email' => $email,
                            'work_phone' => $phone,
                        );
                    }
                    
                    $cont_cnt++;
                    if ($cont_cnt >= 2000)
                    {
                        break;
                    }
                }
            }
            
            if ($cont_cnt >= 2000)
            {
                break;
            }
            $page_nr++;
            
            if ($page_nr >= 10)
            {
                break;
            }
        } while ($hasNext);
        $this->logout();
        
        return $contacts;
    }

    public function logout()
    {
        $sid = isset($this->sid) ? $this->sid : '';
        $this->get('https://navigator-lxa.mail.com/logout?sid=' . $sid);
        return true;
    }

}

?>