<?php

class YahooOauth
{
	public $consumer_key = '';
	public $consumer_secret = '';
	public $debug = 0;
	
	function __construct($consumer_key, $consumer_secret)
	{
		$this->consumer_key = $consumer_key;
		$this->consumer_secret = $consumer_secret;
	}
	
	public function logit($msg, $preamble = true)
	{
		//  date_default_timezone_set('America/Los_Angeles');
		$now = date(DateTime::ISO8601, time());
		error_log(($preamble ? "+++${now}:" : '') . $msg);
	}
	
	/**
	 * Do an HTTP GET
	 * @param string $url
	 * @param int $port (optional)
	 * @param array $headers an array of HTTP headers (optional)
	 * @return array ($info, $header, $response) on success or empty array on error.
	 */
	function do_get($url, $port = 80, $headers = NULL)
	{
		$retarr = array();  // Return value
	
		$curl_opts = array(CURLOPT_URL => $url,
			CURLOPT_PORT => $port,
			CURLOPT_POST => false,
			CURLOPT_SSL_VERIFYHOST => false,
			CURLOPT_SSL_VERIFYPEER => false,
			CURLOPT_RETURNTRANSFER => true);
	
		if ($headers)
		{
			$curl_opts[CURLOPT_HTTPHEADER] = $headers;
		}
	
		$response = $this->do_curl($curl_opts);
	
		if (!empty($response))
		{
			$retarr = $response;
		}
	
		return $retarr;
	}
	
	/**
	 * Do an HTTP POST
	 * @param string $url
	 * @param int $port (optional)
	 * @param array $headers an array of HTTP headers (optional)
	 * @return array ($info, $header, $response) on success or empty array on error.
	 */
	function do_post($url, $postbody, $port = 80, $headers = NULL)
	{
		$retarr = array();  // Return value
	
		$curl_opts = array(CURLOPT_URL => $url,
			CURLOPT_PORT => $port,
			CURLOPT_POST => true,
			CURLOPT_SSL_VERIFYHOST => false,
			CURLOPT_SSL_VERIFYPEER => false,
			CURLOPT_POSTFIELDS => $postbody,
			CURLOPT_RETURNTRANSFER => true);
	
		if ($headers)
		{
			$curl_opts[CURLOPT_HTTPHEADER] = $headers;
		}
	
		$response = $this->do_curl($curl_opts);
	
		if (!empty($response))
		{
			$retarr = $response;
		}
	
		return $retarr;
	}
	
	/**
	 * Make a curl call with given options.
	 * @param array $curl_opts an array of options to curl
	 * @return array ($info, $header, $response) on success or empty array on error.
	 */
	function do_curl($curl_opts)
	{
		$retarr = array();  // Return value
	
		if (!$curl_opts)
		{
			if ($this->debug)
			{
				$this->logit("do_curl:ERR:curl_opts is empty");
			}
			return $retarr;
		}
	
	
		// Open curl session
		$ch = curl_init();
	
		if (!$ch)
		{
			if ($this->debug)
			{
				$this->logit("do_curl:ERR:curl_init failed");
			}
			return $retarr;
		}
	
		// Set curl options that were passed in
		curl_setopt_array($ch, $curl_opts);
	
		// Ensure that we receive full header
		curl_setopt($ch, CURLOPT_HEADER, true);
	
		if ($this->debug)
		{
			curl_setopt($ch, CURLINFO_HEADER_OUT, true);
			curl_setopt($ch, CURLOPT_VERBOSE, true);
		}
	
		// Send the request and get the response
		ob_start();
		$response = curl_exec($ch);
		$curl_spew = ob_get_contents();
		ob_end_clean();
		if ($this->debug && $curl_spew)
		{
			$this->logit("do_curl:INFO:curl_spew begin");
			$this->logit($curl_spew, false);
			$this->logit("do_curl:INFO:curl_spew end");
		}
	
		// Check for errors
		if (curl_errno($ch))
		{
			$errno = curl_errno($ch);
			$errmsg = curl_error($ch);
			if ($this->debug)
			{
				$this->logit("do_curl:ERR:$errno:$errmsg");
			}
			curl_close($ch);
			unset($ch);
			return $retarr;
		}
	
		if ($this->debug)
		{
			$this->logit("do_curl:DBG:header sent begin");
			$header_sent = curl_getinfo($ch, CURLINFO_HEADER_OUT);
			$this->logit($header_sent, false);
			$this->logit("do_curl:DBG:header sent end");
		}
	
		// Get information about the transfer
		$info = curl_getinfo($ch);
	
		// Parse out header and body
		$header_size = curl_getinfo($ch, CURLINFO_HEADER_SIZE);
		$header = substr($response, 0, $header_size);
		$body = substr($response, $header_size);
	
		// Close curl session
		curl_close($ch);
		unset($ch);
	
		if ($this->debug)
		{
			$this->logit("do_curl:DBG:response received begin");
			if (!empty($response))
			{
				$this->logit($response, false);
			}
			$this->logit("do_curl:DBG:response received end");
		}
	
		// Set return value
		array_push($retarr, $info, $header, $body);
	
		return $retarr;
	}
	
	/**
	 * Pretty print some JSON
	 * @param string $json The packed JSON as a string
	 * @param bool $html_output true if the output should be escaped
	 * (for use in HTML)
	 * @link http://us2.php.net/manual/en/function.json-encode.php#80339
	 */
	function json_pretty_print($json, $html_output = false)
	{
		$spacer = '  ';
		$level = 1;
		$indent = 0; // current indentation level
		$pretty_json = '';
		$in_string = false;
	
		$len = strlen($json);
	
		for ($c = 0; $c < $len; $c++)
		{
			$char = $json[$c];
			switch ($char)
			{
				case '{':
				case '[':
					if (!$in_string)
					{
						$indent += $level;
						$pretty_json .= $char . "\n" . str_repeat($spacer, $indent);
					}
					else
					{
						$pretty_json .= $char;
					}
					break;
				case '}':
				case ']':
					if (!$in_string)
					{
						$indent -= $level;
						$pretty_json .= "\n" . str_repeat($spacer, $indent) . $char;
					}
					else
					{
						$pretty_json .= $char;
					}
					break;
				case ',':
					if (!$in_string)
					{
						$pretty_json .= ",\n" . str_repeat($spacer, $indent);
					}
					else
					{
						$pretty_json .= $char;
					}
					break;
				case ':':
					if (!$in_string)
					{
						$pretty_json .= ": ";
					}
					else
					{
						$pretty_json .= $char;
					}
					break;
				case '"':
					if ($c > 0 && $json[$c - 1] != '\\')
					{
						$in_string = !$in_string;
					}
				default:
					$pretty_json .= $char;
					break;
			}
		}
	
		return ($html_output) ?
				'<pre>' . htmlentities($pretty_json) . '</pre>' :
				$pretty_json . "\n";
	}
	
	function get_contacts($guid, $access_token, $access_token_secret, $usePost = false, $passOAuthInHeader = true)
	{
		$retarr = array();  // return value
		$response = array();
	
		$url = 'https://social.yahooapis.com/v1/user/' . $guid . '/contacts;count=1000';
		$params['format'] = 'json';
		$params['view'] = 'compact';
		$params['oauth_version'] = '1.0';
		$params['oauth_nonce'] = mt_rand();
		$params['oauth_timestamp'] = time();
		$params['oauth_consumer_key'] = $this->consumer_key;
		$params['oauth_token'] = $access_token;
	
		// compute hmac-sha1 signature and add it to the params list
		$params['oauth_signature_method'] = 'HMAC-SHA1';
		$params['oauth_signature'] = OAuthHelper::oauth_compute_hmac_sig($usePost ? 'POST' : 'GET', $url, $params, $this->consumer_secret, $access_token_secret);
	
		// Pass OAuth credentials in a separate header or in the query string
		if ($passOAuthInHeader)
		{
			$query_parameter_string = OAuthHelper::oauth_http_build_query($params, true);
			$header = OAuthHelper::build_oauth_header($params, "yahooapis.com");
			$headers[] = $header;
		}
		else
		{
			$query_parameter_string = OAuthHelper::oauth_http_build_query($params);
		}
	
		// POST or GET the request
		if ($usePost)
		{
			$request_url = $url;
			$this->logit("callcontact:INFO:request_url:$request_url");
			$this->logit("callcontact:INFO:post_body:$query_parameter_string");
			$headers[] = 'Content-Type: application/x-www-form-urlencoded';
			$response = $this->do_post($request_url, $query_parameter_string, 443, $headers);
		}
		else
		{
			$request_url = $url . ($query_parameter_string ?
							('?' . $query_parameter_string) : '' );
			$this->logit("callcontact:INFO:request_url:$request_url");
			$response = $this->do_get($request_url, 443, $headers);
		}
		
		if (!empty($response))
		{
			list($info, $header, $body) = $response;
			if ($body)
			{
				$retarr = json_decode($body);
			}
		}
	
		return $retarr;
	}
	
	function get_access_token($request_token, $request_token_secret, $oauth_verifier, $usePost = false, $useHmacSha1Sig = true, $passOAuthInHeader = true)
	{
		$retarr = array();  // return value
		$response = array();
		$url = 'https://api.login.yahoo.com/oauth/v2/get_token';
		$params['oauth_version'] = '1.0';
		$params['oauth_nonce'] = mt_rand();
		$params['oauth_timestamp'] = time();
		$params['oauth_consumer_key'] = $this->consumer_key;
		$params['oauth_token'] = $request_token;
		$params['oauth_verifier'] = $oauth_verifier;
	
		// compute signature and add it to the params list
		if ($useHmacSha1Sig)
		{
			$params['oauth_signature_method'] = 'HMAC-SHA1';
			$params['oauth_signature'] = OAuthHelper::oauth_compute_hmac_sig($usePost ? 'POST' : 'GET', $url, $params, $this->consumer_secret, $request_token_secret);
		}
		else
		{
			$params['oauth_signature_method'] = 'PLAINTEXT';
			$params['oauth_signature'] = OAuthHelper::oauth_compute_plaintext_sig($this->consumer_secret, $request_token_secret);
		}
	
		// Pass OAuth credentials in a separate header or in the query string
		if ($passOAuthInHeader)
		{
			$query_parameter_string = OAuthHelper::oauth_http_build_query($params, false);
			$header = OAuthHelper::build_oauth_header($params, "yahooapis.com");
			$headers[] = $header;
		}
		else
		{
			$query_parameter_string = OAuthHelper::oauth_http_build_query($params);
		}
	
		// POST or GET the request
		if ($usePost)
		{
			$request_url = $url;
			$this->logit("getacctok:INFO:request_url:$request_url");
			$this->logit("getacctok:INFO:post_body:$query_parameter_string");
			$headers[] = 'Content-Type: application/x-www-form-urlencoded';
			$response = $this->do_post($request_url, $query_parameter_string, 443, $headers);
		}
		else
		{
			$request_url = $url . ($query_parameter_string ?
							('?' . $query_parameter_string) : '' );
			$this->logit("getacctok:INFO:request_url:$request_url");
			$response = $this->do_get($request_url, 443, $headers);
		}
	
		// extract successful response
		if (!empty($response))
		{
			list($info, $header, $body) = $response;
			$body_parsed = OAuthHelper::oauth_parse_str($body);
			if (!empty($body_parsed))
			{
				$this->logit("getacctok:INFO:response_body_parsed:");
				// print_r($body_parsed);
			}
			$retarr = $response;
			$retarr[] = $body_parsed;
		}
	
		return $retarr;
	}
	
	/* Function added for getting the request token */
	function get_request_token($callback, $usePost = false, $useHmacSha1Sig = true, $passOAuthInHeader = false)
	{
		$retarr = array();  // return value
		$response = array();
	
		$url = 'https://api.login.yahoo.com/oauth/v2/get_request_token';
		$params['oauth_version'] = '1.0';
		$params['oauth_nonce'] = mt_rand();
		$params['oauth_timestamp'] = time();
		$params['oauth_consumer_key'] = $this->consumer_key;
		$params['oauth_callback'] = $callback;
	
		// compute signature and add it to the params list
		if ($useHmacSha1Sig)
		{
			$params['oauth_signature_method'] = 'HMAC-SHA1';
			$params['oauth_signature'] = OAuthHelper::oauth_compute_hmac_sig($usePost ? 'POST' : 'GET', $url, $params, $this->consumer_secret, null);
		}
		else
		{
			$params['oauth_signature_method'] = 'PLAINTEXT';
			$params['oauth_signature'] = OAuthHelper::oauth_compute_plaintext_sig($this->consumer_secret, null);
		}
	
		// Pass OAuth credentials in a separate header or in the query string
		if ($passOAuthInHeader)
		{
	
			$query_parameter_string = OAuthHelper::oauth_http_build_query($params, FALSE);
	
			$header = OAuthHelper::build_oauth_header($params, "yahooapis.com");
			$headers[] = $header;
		}
		else
		{
			$query_parameter_string = OAuthHelper::oauth_http_build_query($params);
		}
	
		// POST or GET the request
		if ($usePost)
		{
			$request_url = $url;
			$this->logit("getreqtok:INFO:request_url:$request_url");
			$this->logit("getreqtok:INFO:post_body:$query_parameter_string");
			$headers[] = 'Content-Type: application/x-www-form-urlencoded';
			$response = $this->do_post($request_url, $query_parameter_string, 443, $headers);
		}
		else
		{
			$request_url = $url . ($query_parameter_string ?
							('?' . $query_parameter_string) : '' );
	
			$this->logit("getreqtok:INFO:request_url:$request_url");
	
			$response = $this->do_get($request_url, 443, $headers);
		}
	
		// extract successful response
		if (!empty($response))
		{
			list($info, $header, $body) = $response;
			$body_parsed = OAuthHelper::oauth_parse_str($body);
			if (!empty($body_parsed))
			{
				$this->logit("getreqtok:INFO:response_body_parsed:");
				//print_r($body_parsed);
			}
			$retarr = $response;
			$retarr[] = $body_parsed;
		}
	
		return $retarr;
	}
		
}

?>